@@ -44,11 +44,11 @@ use constant	DEFAULT_MAX_SLURP_SIZE => 16 * 1024;	# CSV files <= than this size
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =cut
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 =head1 SYNOPSIS
 
@@ -56,7 +56,7 @@ Abstract class giving read-only access to CSV, XML and SQLite databases via Perl
 Look for databases in $directory in this order:
 1) SQLite (file ends with .sql)
 2) PSV (pipe separated file, file ends with .psv)
-3) CSV (file ends with .csv or .db, can be gzipped)
+3) CSV (file ends with .csv or .db, can be gzipped) (note the default sep_char is '!' not ',')
 4) XML (file ends with .xml)
 
 For example, you can access the files in /var/db/foo.csv via this class:
@@ -79,6 +79,15 @@ entries are keyed on that and sorts are based on it.
 To turn that off, pass 'no_entry' to the constructor, for legacy
 reasons it's enabled by default.
 
+    # Regular CSV: There is no entry column and the separators are commas
+    sub new
+    {
+	my $class = shift;
+	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+
+	return $class->SUPER::new(no_entry => 1, sep_char => ',', %args);
+    }
+
 CSV files that are not no_entry can have empty lines or comment lines starting with '#',
 to make them more readable.
 
@@ -131,19 +140,17 @@ If the arguments are not set, tries to take from class level defaults.
 =cut
 
 sub new {
-	my $proto = shift;
+	my $class = shift;
 	my %args;
 
 	if(ref($_[0]) eq 'HASH') {
 		%args = %{$_[0]};
-	} elsif(scalar(@_) % 2 == 0) {
+	} elsif((scalar(@_) % 2) == 0) {
 		%args = @_;
 	} elsif(scalar(@_) == 1) {
 		$args{'directory'} = shift;
 	}
 
-	my $class = ref($proto) || $proto;
-
 	if(!defined($class)) {
 		# Using Database::Abstraction->new(), not Database::Abstraction::new()
 		carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
@@ -168,7 +175,8 @@ sub new {
 		# table => $args{'table'},	# The name of the file containing the table, defaults to the class name
 		# no_entry => $args{'no_entry'} || 0,
 	# }, $class;
-	# Reseen keys take precedence, so defaults come first
+
+	# Re-seen keys take precedence, so defaults come first
 	return bless {
 		no_entry => 0,
 		cache_duration => '1 hour',
@@ -187,19 +195,10 @@ Pass a class that will be used for logging.
 sub set_logger
 {
 	my $self = shift;
+	my $args = $self->_get_params('logger', @_);
 
-	my %args;
-
-	if(ref($_[0]) eq 'HASH') {
-		%args = %{$_[0]};
-	} elsif(scalar(@_) % 2 == 0) {
-		%args = @_;
-	} elsif((scalar(@_) == 1) && ref($_[0])) {
-		$args{'logger'} = shift;
-	}
-
-	if(defined($args{'logger'})) {
-		$self->{'logger'} = $args{'logger'};
+	if(defined($args->{'logger'})) {
+		$self->{'logger'} = $args->{'logger'};
 		return $self;
 	}
 	Carp::croak('Usage: set_logger(logger => $logger)')
@@ -207,12 +206,11 @@ sub set_logger
 
 # Open the database.
 
-# FIXME: The default separator character is (for my historical reasons) '!' not ','
-
 sub _open {
 	my $self = shift;
+	my $sep_char = ($self->{'sep_char'} ? $self->{'sep_char'} : '!');
 	my %args = (
-		sep_char => '!',
+		sep_char => $sep_char,
 		((ref($_[0]) eq 'HASH') ? %{$_[0]} : @_)
 	);
 
@@ -270,7 +268,7 @@ sub _open {
 		}
 		if(defined($slurp_file) && (-r $slurp_file)) {
 			close($fin);
-			my $sep_char = $args{'sep_char'};
+			$sep_char = $args{'sep_char'};
 			if($self->{'logger'}) {
 				$self->{'logger'}->debug(__LINE__, ' of ', __PACKAGE__, ": slurp_file = $slurp_file, sep_char = $sep_char");
 			}
@@ -429,21 +427,21 @@ Returns an array of hash references
 sub selectall_hash
 {
 	my $self = shift;
-	my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+	my $params = $self->_get_params(undef, @_);
 
 	my $table = $self->{table} || ref($self);
 	$table =~ s/.*:://;
 
 	if($self->{'data'}) {
-		if(scalar(keys %params) == 0) {
+		if(scalar(keys %${params}) == 0) {
 			if($self->{'logger'}) {
 				$self->{'logger'}->trace("$table: selectall_hash fast track return");
 			}
 			return values %{$self->{'data'}};
 			# my @rc = values %{$self->{'data'}};
 			# return @rc;
-		} elsif((scalar(keys %params) == 1) && defined($params{'entry'}) && !$self->{'no_entry'}) {
-			return $self->{'data'}->{$params{'entry'}};
+		} elsif((scalar(keys %{$params}) == 1) && defined($params->{'entry'}) && !$self->{'no_entry'}) {
+			return $self->{'data'}->{$params->{'entry'}};
 		}
 	}
 
@@ -460,8 +458,8 @@ sub selectall_hash
 	}
 
 	my @query_args;
-	foreach my $c1(sort keys(%params)) {	# sort so that the key is always the same
-		my $arg = $params{$c1};
+	foreach my $c1(sort keys(%{$params})) {	# sort so that the key is always the same
+		my $arg = $params->{$c1};
 		if(ref($arg)) {
 			if($self->{'logger'}) {
 				$self->{'logger'}->fatal("selectall_hash $query: argument is not a string");
@@ -562,20 +560,20 @@ which is worked out from the class name
 
 sub fetchrow_hashref {
 	my $self = shift;
-	my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+	my $params = $self->_get_params(undef, @_);
 
-	my $table = $params{'table'} || $self->{'table'} || ref($self);
+	my $table = $params->{'table'} || $self->{'table'} || ref($self);
 	$table =~ s/.*:://;
 
-	if($self->{'data'} && (!$self->{'no_entry'}) && (scalar keys(%params) == 1) && defined($params{'entry'})) {
+	if($self->{'data'} && (!$self->{'no_entry'}) && (scalar keys(%{$params}) == 1) && defined($params->{'entry'})) {
 		if(my $logger = $self->{'logger'}) {
 			$logger->debug('Fast return from slurped data');
 		}
-		return $self->{'data'}->{$params{'entry'}};
+		return $self->{'data'}->{$params->{'entry'}};
 	}
 
 	my $query = 'SELECT * FROM ';
-	if(my $t = delete $params{'table'}) {
+	if(my $t = delete $params->{'table'}) {
 		$query .= $t;
 	} else {
 		$query .= $table;
@@ -589,8 +587,8 @@ sub fetchrow_hashref {
 		$done_where = 1;
 	}
 	my @query_args;
-	foreach my $c1(sort keys(%params)) {	# sort so that the key is always the same
-		if(my $arg = $params{$c1}) {
+	foreach my $c1(sort keys(%{$params})) {	# sort so that the key is always the same
+		if(my $arg = $params->{$c1}) {
 			my $keyword;
 
 				if(ref($arg)) {
@@ -676,24 +674,16 @@ to the query.
 
 sub execute {
 	my $self = shift;
-	my %args;
+	my $args = $self->_get_params('query', @_);
 
-	if(ref($_[0]) eq 'HASH') {
-		%args = %{$_[0]};
-	} elsif(scalar(@_) % 2 == 0) {
-		%args = @_;
-	} elsif((scalar(@_) == 1) && !ref($_[0])) {
-		$args{'query'} = shift;
-	}
-
-	Carp::croak(__PACKAGE__, ': Usage: execute(query => $query)') unless(defined($args{'query'}));
+	Carp::croak(__PACKAGE__, ': Usage: execute(query => $query)') unless(defined($args->{'query'}));
 
 	my $table = $self->{table} || ref($self);
 	$table =~ s/.*:://;
 
 	$self->_open() if(!$self->{$table});
 
-	my $query = $args{'query'};
+	my $query = $args->{'query'};
 	if($query !~ / FROM /i) {
 		$query .= " FROM $table";
 	}
@@ -917,6 +907,41 @@ sub DESTROY {
 	}
 }
 
+# Helper routine to parse the arguments given to a function,
+#	allowing the caller to call the function in anyway that they want
+#	e.g. foo('bar'), foo(arg => 'bar'), foo({ arg => 'bar' }) all mean the same
+#	when called _get_params('arg', @_);
+sub _get_params
+{
+	shift;
+	my $default = shift;
+
+	if(ref($_[0]) eq 'HASH') {
+		# %rc = %{$_[0]};
+		return $_[0];
+	}
+
+	my %rc;
+
+	if((scalar(@_) % 2) == 0) {
+		%rc = @_;
+	} elsif(scalar(@_) == 1) {
+		if(defined($default)) {
+			$rc{$default} = shift;
+		} else {
+			my @c = caller(1);
+			my $func = $c[3];	# calling function name
+			Carp::croak('Usage: ', __PACKAGE__, "->$func()");
+		}
+	} elsif((scalar(@_) == 0) && defined($default)) {
+		my @c = caller(1);
+		my $func = $c[3];	# calling function name
+		Carp::croak('Usage: ', __PACKAGE__, "->$func($default => " . '$val)');
+	}
+
+	return \%rc;
+}
+
 =head1 AUTHOR
 
 Nigel Horne, C<< <njh at bandsman.co.uk> >>
